// CodeWarServerView.cpp : implementation of the CCodeWarServerView class
//

#include "stdafx.h"
#include "CodeWarServer.h"

#include "CodeWarServerDoc.h"
#include "CodeWarServerView.h"

#include "D3DSphere.h"
#include "D3DBox.h"
#include "D3DCamera.h"
D3DCamera camera;

UINT const MyRenderTimer = 12345;
UINT const MyRenderTime = 20;

#include "Arena.h"
#include "Square.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

extern IDirect3D8 * g_pD3D;
extern IDirect3DDevice8 * g_pd3dDevice;

// CCodeWarServerView

IMPLEMENT_DYNCREATE(CCodeWarServerView, CFormView)

BEGIN_MESSAGE_MAP(CCodeWarServerView, CFormView)
    ON_BN_CLICKED(IDC_LISTEN_BTN, OnBnClickedListenBtn)
    ON_WM_TIMER()
    ON_WM_DESTROY()
//    ON_WM_LBUTTONDOWN()
//    ON_WM_LBUTTONUP()
//    ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

// CCodeWarServerView construction/destruction

CCodeWarServerView::CCodeWarServerView()
	: CFormView(CCodeWarServerView::IDD)
    , m_Port(1234)
//    , m_lbuttondown( false )
{
	// TODO: add construction code here

}

CCodeWarServerView::~CCodeWarServerView()
{
}

void CCodeWarServerView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    DDX_Text(pDX, IDC_PORT, m_Port);
}

BOOL CCodeWarServerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CCodeWarServerView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

    D3DDISPLAYMODE d3ddm;
    if( FAILED( g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return;

    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed   = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
//    d3dpp.hDeviceWindow = m_pMainWnd->m_hWnd;

    GetDlgItem( IDC_FRAME, &d3dpp.hDeviceWindow );

    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
        return;

    g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
    g_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME); // D3DFILL_POINT, D3DFILL_WIREFRAME or D3DFILL_SOLID

    D3DXMATRIX mProjection;
    D3DXMatrixPerspectiveFovLH( &mProjection, 45.0f, 640.0f / 480.0f, 1.0f, -100.0f );
    g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &mProjection );

    camera.create( g_pd3dDevice );

    SetTimer( MyRenderTimer, MyRenderTime, NULL );
}


// CCodeWarServerView diagnostics

#ifdef _DEBUG
void CCodeWarServerView::AssertValid() const
{
	CFormView::AssertValid();
}

void CCodeWarServerView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CCodeWarServerDoc* CCodeWarServerView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCodeWarServerDoc)));
	return (CCodeWarServerDoc*)m_pDocument;
}
#endif //_DEBUG


// CCodeWarServerView message handlers

void CCodeWarServerView::OnBnClickedListenBtn()
{
    // Tell the application to create CodeWar and start listening
    // for incoming registrations from robots.
    static bool initialized = false;

    if( initialized )
    {
        return;
    }

    UpdateData( TRUE );

    if( !theApp.m_CodeWar.initialize( 9, 9, m_Port ) )
    {
        AfxMessageBox("Could not initialize CodeWar.");
    }

    // Create D3DBox objects so that the Arena can be visualized
    Arena * arena = theApp.m_CodeWar.getArena();

    if( arena )
    {
        XType width = arena->getWidth();
        YType height = arena->getHeight();

        // Resize the vectors
        m_Arena.resize( width );

        for( YType i=0; i<width; ++i )
        {
            m_Arena[i].resize( height );
        }

        for( XType x=0; x<width; ++x )
        {
            for( YType y=0; y<height; ++y )
            {
                Square * square = arena->getSquare( x, y );

                if( square )
                {
                    D3DBox * box = new D3DBox();

                    float const multiplier = 1.0f;

                    box->create( g_pd3dDevice );
                    box->move( (float)x * multiplier, (float)y * multiplier, 20.0f );
                    box->scale( multiplier, multiplier, multiplier );

                    switch( square->getType() )
                    {
                    case Square::Empty:  box->setColor( 1.0f, 1.0f, 0.0f ); break; // Yellow
                    case Square::Border: box->setColor( 1.0f, 1.0f, 1.0f ); break; // White
                    case Square::Food:   box->setColor( 0.0f, 1.0f, 0.0f ); break; // Green
                    case Square::Life:   box->setColor( 0.0f, 0.0f, 1.0f ); break; // Blue
                    }

                    m_Arena[x][y] = box;
                }
            } // for(y)
        } // for(x)

        //******************************************************************
        // TODO: Adjust the view so that the complete arena is in sight
        //******************************************************************

        initialized = true;
    } // if( arena )
}

void CCodeWarServerView::render(void)
{
    // Clear the back buffer to a blue color
    if( g_pd3dDevice )
    {
        g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );

        // Begin the scene.
        g_pd3dDevice->BeginScene();

        // Rendering of scene objects happens here.
        camera.updateView();

        // Start with drawing the arena
        Arena * arena = theApp.m_CodeWar.getArena();

        if( arena )
        {
            XType width = arena->getWidth();
            YType height = arena->getHeight();

            for( XType x=0; x<width; ++x )
            {
                for( YType y=0; y<height; ++y )
                {
                    Square * square = arena->getSquare( x, y );

                    if( square )
                    {
                        std::list<Robot *> robots;
                        square->getRobots( robots );
                        if( !robots.empty() )
                        {
                            float const multiplier = 1.0f;
                            D3DSphere sphere;
                            sphere.create( g_pd3dDevice );

if( robots.front()->m_dead )
sphere.changeColor( RGB(255, 0, 0) );
else
sphere.changeColor( RGB(255, 255, 0) );

                            sphere.move( (float)x * multiplier, (float)y * multiplier, 20.0f );
                            sphere.scale( multiplier/2, multiplier/2, multiplier/2 );
                            sphere.draw();
/*
                    float const multiplier = 1.0f;

                    box->create( g_pd3dDevice );
                    box->move( (float)x * multiplier, (float)y * multiplier, 20.0f );
                    box->scale( multiplier, multiplier, multiplier );
*/
                        }

                    }

                    D3DBox * box = m_Arena[x][y];

                    box->draw();
                }
            }
        }

        // End the scene.
        g_pd3dDevice->EndScene();

        g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
    }
}

void CCodeWarServerView::OnTimer(UINT nIDEvent)
{
    // TODO: Add your message handler code here and/or call default
    switch( nIDEvent )
    {
    case MyRenderTimer:
        KillTimer( MyRenderTimer );
        render();
        SetTimer( MyRenderTimer, MyRenderTime, NULL );
        break;
    }

    CFormView::OnTimer(nIDEvent);
}

void CCodeWarServerView::OnDestroy()
{
    KillTimer( MyRenderTimer );

    Arena * arena = theApp.m_CodeWar.getArena();

    if( arena )
    {
        XType width = arena->getWidth();
        YType height = arena->getHeight();

        for( XType x=0; x<width; ++x )
        {
            for( YType y=0; y<height; ++y )
            {
                D3DBox * box = m_Arena[x][y];

                if( box )
                {
                    delete box;
                }
            }
        }

        m_Arena.clear();
    }

    CFormView::OnDestroy();

    // TODO: Add your message handler code here
}

BOOL CCodeWarServerView::PreTranslateMessage(MSG* pMsg)
{
    // TODO: Add your specialized code here and/or call the base class
    CWnd * wnd = GetDlgItem( IDC_FRAME );

    static bool lbuttondown = false;
    static CPoint previous;
    static CPoint current;

    D3DXVECTOR3 tmpLook  = camera.m_vLook;
    D3DXVECTOR3 tmpRight = camera.m_vRight;
    D3DXMATRIX mtxRot;

    switch( pMsg->message )
    {
    case WM_LBUTTONDOWN:
        if( wnd )
        {
            RECT rect;
            wnd->GetWindowRect( &rect );
            ScreenToClient( &rect );

            CPoint point( LOWORD( pMsg->lParam ), HIWORD( pMsg->lParam ) );

            if( (rect.left <= point.x) && (point.x <= rect.right) &&
                (rect.top <= point.y) && (point.y <= rect.bottom) )
            {
                previous = current = point;

                lbuttondown = true;
            }
        }
        break;

    case WM_LBUTTONUP:
        lbuttondown = false;
        break;

    case WM_MOUSEMOVE:
        {
        CPoint point( LOWORD( pMsg->lParam ), HIWORD( pMsg->lParam ) );

        current = point;

        if( lbuttondown )
        {
            int nYDiff = (current.y - previous.y);
            int nXDiff = (current.x - previous.x);

            if( nYDiff != 0 )
            {
                D3DXMatrixRotationAxis( &mtxRot, &camera.m_vRight, D3DXToRadian((float)nYDiff / 3.0f));
                D3DXVec3TransformCoord( &camera.m_vLook, &camera.m_vLook, &mtxRot );
                D3DXVec3TransformCoord( &camera.m_vUp, &camera.m_vUp, &mtxRot );
            }

            if( nXDiff != 0 )
            {
                D3DXMatrixRotationAxis( &mtxRot, &D3DXVECTOR3(0,1,0), D3DXToRadian((float)nXDiff / 3.0f) );
                D3DXVec3TransformCoord( &camera.m_vLook, &camera.m_vLook, &mtxRot );
                D3DXVec3TransformCoord( &camera.m_vUp, &camera.m_vUp, &mtxRot );
            }
        }

        previous = current;
        break;
        }

    case WM_KEYDOWN:
        switch( pMsg->wParam )
        {
        case 38: // Up Arrow Key = View moves forward
            camera.forward();
            break;

        case 40: // Down Arrow Key = View moves backward
            camera.backward();
            break;

        case 37: // Left Arrow Key = View side-steps or strafes to the left
            camera.turnLeft();
            break;

        case 39: // Right Arrow Key = View side-steps or strafes to the right
            camera.turnRight();
            break;

        }
        break;
    }
    return CFormView::PreTranslateMessage(pMsg);
}
