#include "d3dsphere.h"

D3DSphere::D3DSphere( COLORREF color )
: m_pSphereMesh( NULL )
, m_pd3dDevice( NULL )
, m_Color( color )
{
}

D3DSphere::~D3DSphere(void)
{
    if( NULL != m_pSphereMesh )
    {
        m_pSphereMesh->Release();
    }
}

bool D3DSphere::create(IDirect3DDevice8 * pd3dDevice)
{
    if( NULL == pd3dDevice )
    {
        return( false );
    }

    m_pd3dDevice = pd3dDevice;

    if( NULL != m_pSphereMesh )
    {
        m_pSphereMesh->Release();
        m_pSphereMesh = NULL;
    }

    LPD3DXMESH pTempMesh;

    //
    // We'll use the D3DXCreateSphere utility function to create a simple 
    // sphere meshe to experiment with.
    //

    if( FAILED( D3DXCreateSphere( m_pd3dDevice, 1.0f, 20, 20, &pTempMesh, NULL ) ) )
    {
        return( false );
    }

    //
    // Unfortunately, the D3DXCreateSphere utility function creates a mesh 
    // with no color, so we'll need to make a clone of the original meshes 
    // using a FVF code that does include color.
    //
    // Once that's been done, we'll need to set the color values to something 
    // appropriate for our solar system model.
    //

    //
    // Clone the original sphere mesh and make it yellow...
    //

    LPDIRECT3DVERTEXBUFFER8 pTempVertexBuffer;

    pTempMesh->CloneMeshFVF( 0, D3DFVF_XYZ | D3DFVF_DIFFUSE, m_pd3dDevice, &m_pSphereMesh );

    if( SUCCEEDED( m_pSphereMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
    {
        int nNumVerts = m_pSphereMesh->GetNumVertices();
        Vertex *pVertices = NULL;

        pTempVertexBuffer->Lock( 0, 0, (BYTE**)&pVertices, 0 );

        for( int i = 0; i < nNumVerts; ++i )
        {
            pVertices[i].diffuse = D3DCOLOR_RGBA( GetRValue(m_Color), GetGValue(m_Color), GetBValue(m_Color), 255 );
        }

        pTempVertexBuffer->Unlock();

        pTempVertexBuffer->Release();
    }

    //
    // The temp mesh is no longer needed...
    //

    pTempMesh->Release();

    return( true );
}

bool D3DSphere::changeColor( COLORREF color )
{
    m_Color = color;

    return( create( m_pd3dDevice ) );
}

void D3DSphere::move(float x, float y, float z)
{
    D3DXMatrixTranslation( &m_mMove, x, y, z );
}

void D3DSphere::scale(float x, float y, float z)
{
    D3DXMatrixScaling( &m_mScale, x, y, z );
}

bool D3DSphere::draw(void)
{
    if( NULL == m_pd3dDevice )
    {
        return( false );
    }

    //
    // The sphere is easy because the mesh for it is initially created centered  
    // at origin.
    //

    D3DXMATRIX mSunMatrix;

    // Now, concatenate them together...

    mSunMatrix = m_mScale *         // 1. Uniformly scale the Sun up in size
                 m_mMove;           // 2. and move the sun.

    m_pd3dDevice->SetTransform( D3DTS_WORLD, &mSunMatrix );

    if( m_pSphereMesh )
    {
        if( SUCCEEDED( m_pSphereMesh->DrawSubset( 0 ) ) )
        {
            return( true );
        }
    }

    return( false );
}
